#include "config.h"

#include <sstream>
#include <stdexcept>

#include "QueryResultsTimeRange.hh"

namespace /* anonymous */
{
} /* namespace - anonymous */

namespace diskCache
{
  //=====================================================================
  //
  //=====================================================================

  QueryResultsTimeRange::RangeData::
  ~RangeData( )
  {
  }

  //=====================================================================
  //
  //=====================================================================

  const Query::gps_time_type QueryResultsTimeRange::m_RESAMPLE_PADDING = 1;

  QueryResultsTimeRange::
  QueryResultsTimeRange( format_type Format, bool AllowGaps )
    : QueryResults( Format, TYPE_TIME_RANGE ),
      m_allow_gaps( AllowGaps )
  {
  }

  QueryResultsTimeRange::
  ~QueryResultsTimeRange( )
  {
  }

  void QueryResultsTimeRange::
  Append( Query::gps_time_type Start,
	  reconstitute_func_type Func,
	  range_data_type	 Data )
  {
    m_file_data[ Start ] = filename_info( Func, Data );
  }

  void QueryResultsTimeRange::
  Finalize( const Query& SearchParams )
  {
    //-------------------------------------------------------------------
    // Start of data set
    //-------------------------------------------------------------------
    switch( format( ) )
    {
    case FORMAT_TCL:
      tcl_list_begin( m_string_result );
      break;
    default:
      // Do nothing
      break;
    }
    //-------------------------------------------------------------------
    // Place each file into result set
    //-------------------------------------------------------------------
    for ( filename_info_container::const_iterator
	    cur = m_file_data.begin( ),
	    last = m_file_data.end( );
	  cur != last;
	  ++cur )
    {
      (*cur->second.s_func)(cur->first, cur->second.s_data.get( ), *this );
    }
    //-------------------------------------------------------------------
    // End of data set
    //-------------------------------------------------------------------
    switch( format( ) )
    {
    case FORMAT_TCL:
      tcl_list_end( m_string_result );
      break;
    default:
      // Do nothing
      break;
    }
    //-------------------------------------------------------------------
    // Put in conflict data
    //-------------------------------------------------------------------
    tcl_list_begin( m_string_result );
    if ( m_conflicts.empty( ) == false )
    {
      m_string_result == " ";
      m_string_result += m_conflicts;
    }
    if ( ( m_allow_gaps == false )
	 && ( SearchParams.TimeRange( ).empty( ) == false ) )
    {
      //-----------------------------------------------------------------
      // Report the gappy information
      //-----------------------------------------------------------------
      tcl_list_begin( m_string_result );
      m_string_result += "Gaps are detected:";
      for ( Query::time_range_type::const_iterator
	      cur_gap = SearchParams.TimeRange( ).begin( ),
	      last_gap = SearchParams.TimeRange( ).end( );
	    cur_gap != last_gap;
	    ++cur_gap )
      {
	std::ostringstream	gap;
	gap << " " << cur_gap->first << " " << cur_gap->second;

	m_string_result += gap.str( );
      }
      tcl_list_end( m_string_result );
    }
    tcl_list_end( m_string_result );
  }

  bool QueryResultsTimeRange::
  Resample( Query& SearchParams ) const
  {
    if ( (m_file_data.empty( ) ) // No results
	 || ( SearchParams.TimeRange( ).empty( ) == false ) ) // gappy
    {
      //-----------------------------------------------------------------
      // Cannot resample a query that has gaps
      //-----------------------------------------------------------------
      return false;
    }

    //-------------------------------------------------------------------
    // Expand the search by N seconds on each side of the query.
    //-------------------------------------------------------------------
    Query::time_range_type	retval;
    filename_info_container::const_iterator
      head = m_file_data.begin( ),
      tail = m_file_data.end( );

    --tail;

    retval[ head->first - m_RESAMPLE_PADDING ] = head->first;
    retval[ tail->second.s_data->EndTime( ) ] = tail->second.s_data->EndTime( ) + m_RESAMPLE_PADDING;

    SearchParams.TimeRange( retval );

    return true;
  }

} // namespace - diskCache
