#ifndef FrameCPP_VERSION_3_FrRawData_HH
#define FrameCPP_VERSION_3_FrRawData_HH

#include <string>
#include <vector>

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version3/FrameSpec.hh"
#include "framecpp/Version3/FrAdcData.hh"
#include "framecpp/Version3/FrMsg.hh"
#include "framecpp/Version3/FrSerData.hh"
#include "framecpp/Version3/FrVect.hh"
#include "framecpp/Version3/STRING.hh"

namespace FrameCPP
{
  namespace Version_3
  {
    class FrRawData
      : public Common::FrameSpec::Object
    {
    public:
      typedef Common::SearchContainer< FrSerData, &FrSerData::GetName >
      firstSer_type;
      typedef firstSer_type::iterator firstSer_iterator;
      typedef firstSer_type::const_iterator const_firstSer_iterator;
      typedef firstSer_type::hash_iterator firstSer_hash_iterator;
      typedef firstSer_type::const_hash_iterator const_firstSer_hash_iterator;

      typedef Common::SearchContainer< FrAdcData, &FrAdcData::GetName >
      firstAdc_type;
      typedef firstAdc_type::iterator firstAdc_iterator;
      typedef firstAdc_type::const_iterator const_firstAdc_iterator;
      typedef firstAdc_type::hash_iterator firstAdc_hash_iterator;
      typedef firstAdc_type::const_hash_iterator const_firstAdc_hash_iterator;

      typedef Common::SearchContainer< FrMsg, &FrMsg::GetAlarm >
      logMsg_type;
      typedef logMsg_type::iterator logMsg_iterator;
      typedef logMsg_type::const_iterator const_logMsg_iterator;
      typedef logMsg_type::hash_iterator logMsg_hash_iterator;
      typedef logMsg_type::const_hash_iterator const_logMsg_hash_iterator;

      typedef Common::SearchContainer< FrVect, &FrVect::GetName >
      more_type;
      typedef more_type::iterator more_iterator;
      typedef more_type::const_iterator const_more_iterator;
      typedef more_type::hash_iterator more_hash_iterator;
      typedef more_type::const_hash_iterator const_more_hash_iterator;

      //: Default constructor
      FrRawData( );
      FrRawData( const FrRawData& rawData );
      FrRawData( const std::string& name );
      ~FrRawData( );

      virtual cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrRawData* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      //: Write the structure to the stream
      virtual void Write( Common::OStream& Stream ) const;

      const std::string& GetName() const;
      const firstSer_type& RefFirstSer( ) const;
      firstSer_type& RefFirstSer( );
      const firstAdc_type& RefFirstAdc( ) const;
      firstAdc_type&  RefFirstAdc( );
      const logMsg_type& RefLogMsg( ) const;
      logMsg_type& RefLogMsg( );
      const more_type& RefMore( ) const;
      more_type& RefMore( );
   
      static const Common::FrameSpec::Info::frame_object_types CLASS_ID
      = Common::FrameSpec::Info::FSI_FR_RAW_DATA;

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      /// \brief comparison operator
      bool operator==( const FrRawData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      struct fr_raw_data_type {
	fr_raw_data_type( );
	bool operator==( const fr_raw_data_type& RHS ) const;

	STRING		name;
	firstSer_type	firstSer;
	firstAdc_type	firstAdc;
	logMsg_type	logMsg;
	more_type	more;
      };

      FrRawData( Common::IStream& Stream );

      fr_raw_data_type	m_data;
    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrRawData::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Common::PromoteObject( DATA_FORMAT_VERSION, Source, Obj );
    }

    inline const char* const FrRawData::
    StructName( )
    {
      static const char* const name = "FrRawData";
      return name;
    }

    inline const FrRawData::firstSer_type& FrRawData::
    RefFirstSer( ) const
    {
      return m_data.firstSer;
    }

    inline FrRawData::firstSer_type& FrRawData::
    RefFirstSer( )
    {
      return m_data.firstSer;
    }

    inline const FrRawData::firstAdc_type& FrRawData::
    RefFirstAdc( ) const
    {
      return m_data.firstAdc;
    }

    inline FrRawData::firstAdc_type&  FrRawData::
    RefFirstAdc( )
    {
      return m_data.firstAdc;
    }

    inline const FrRawData::logMsg_type& FrRawData::
    RefLogMsg( ) const
    {
      return m_data.logMsg;
    }

    inline FrRawData::logMsg_type& FrRawData::
    RefLogMsg( )
    {
      return m_data.logMsg;
    }

    inline const FrRawData::more_type& FrRawData::
    RefMore( ) const
    {
      return m_data.more;
    }

    inline FrRawData::more_type& FrRawData::
    RefMore( )
    {
      return m_data.more;
    }
   
    inline bool FrRawData::
    operator==( const FrRawData& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }

    inline bool FrRawData::fr_raw_data_type::
    operator==( const fr_raw_data_type& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( name == RHS.name )
		    && ( firstSer == RHS.firstSer )
		    && ( firstAdc == RHS.firstAdc )
		    && ( logMsg == RHS.logMsg )
		    && ( more == RHS.more )
		    )
	       );
    }
  } // namespace - Version_3
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_3_FrRawData_HH */
